These three functions follow the same pattern. To deduplicate the code,
let's introduce a common helper __kmemdup_nul().

Suggested-by: Andrew Morton <a...@linux-foundation.org>
Signed-off-by: Yafang Shao <laoar.s...@gmail.com>
Cc: Simon Horman <ho...@kernel.org>
Cc: Matthew Wilcox <wi...@infradead.org>
---
 mm/util.c | 67 +++++++++++++++++++++----------------------------------
 1 file changed, 26 insertions(+), 41 deletions(-)

diff --git a/mm/util.c b/mm/util.c
index 4542d8a800d9..310c7735c617 100644
--- a/mm/util.c
+++ b/mm/util.c
@@ -45,33 +45,40 @@ void kfree_const(const void *x)
 EXPORT_SYMBOL(kfree_const);
 
 /**
- * kstrdup - allocate space for and copy an existing string
- * @s: the string to duplicate
+ * __kmemdup_nul - Create a NUL-terminated string from @s, which might be 
unterminated.
+ * @s: The data to copy
+ * @len: The size of the data, including the null terminator
  * @gfp: the GFP mask used in the kmalloc() call when allocating memory
  *
  * Return: newly allocated copy of @s or %NULL in case of error
  */
-noinline
-char *kstrdup(const char *s, gfp_t gfp)
+static __always_inline char *__kmemdup_nul(const char *s, size_t len, gfp_t 
gfp)
 {
-       size_t len;
        char *buf;
 
-       if (!s)
+       buf = kmalloc_track_caller(len, gfp);
+       if (!buf)
                return NULL;
 
-       len = strlen(s) + 1;
-       buf = kmalloc_track_caller(len, gfp);
-       if (buf) {
-               memcpy(buf, s, len);
-               /* During memcpy(), the string might be updated to a new value,
-                * which could be longer than the string when strlen() is
-                * called. Therefore, we need to add a null termimator.
-                */
-               buf[len - 1] = '\0';
-       }
+       memcpy(buf, s, len);
+       /* Ensure the buf is always NUL-terminated, regardless of @s. */
+       buf[len - 1] = '\0';
        return buf;
 }
+
+/**
+ * kstrdup - allocate space for and copy an existing string
+ * @s: the string to duplicate
+ * @gfp: the GFP mask used in the kmalloc() call when allocating memory
+ *
+ * Return: newly allocated copy of @s or %NULL in case of error
+ */
+noinline
+char *kstrdup(const char *s, gfp_t gfp)
+{
+       return s ? __kmemdup_nul(s, strlen(s) + 1, gfp) : NULL;
+}
 EXPORT_SYMBOL(kstrdup);
 
 /**
@@ -106,19 +113,7 @@ EXPORT_SYMBOL(kstrdup_const);
  */
 char *kstrndup(const char *s, size_t max, gfp_t gfp)
 {
-       size_t len;
-       char *buf;
-
-       if (!s)
-               return NULL;
-
-       len = strnlen(s, max);
-       buf = kmalloc_track_caller(len+1, gfp);
-       if (buf) {
-               memcpy(buf, s, len);
-               buf[len] = '\0';
-       }
-       return buf;
+       return s ? __kmemdup_nul(s, strnlen(s, max) + 1, gfp) : NULL;
 }
 EXPORT_SYMBOL(kstrndup);
 
@@ -192,17 +187,7 @@ EXPORT_SYMBOL(kvmemdup);
  */
 char *kmemdup_nul(const char *s, size_t len, gfp_t gfp)
 {
-       char *buf;
-
-       if (!s)
-               return NULL;
-
-       buf = kmalloc_track_caller(len + 1, gfp);
-       if (buf) {
-               memcpy(buf, s, len);
-               buf[len] = '\0';
-       }
-       return buf;
+       return s ? __kmemdup_nul(s, len + 1, gfp) : NULL;
 }
 EXPORT_SYMBOL(kmemdup_nul);
 
@@ -45,33 +45,40 @@ void kfree_const(const void *x)
  * @gfp: the GFP mask used in the kmalloc() call when allocating memory
  *
- * Return: newly allocated copy of @s or %NULL in case of error
+ * Return: newly allocated copy of @s with NUL-termination or %NULL in
+ * case of error
  */
-- 
2.43.5

Reply via email to